from django.conf import settings
from django.conf.urls import url
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from wagtail.admin.menu import MenuItem
from wagtail.core import hooks
from wagtail.users.widgets import UserListingButton

from wagtail_2fa import views


@hooks.register('register_admin_urls')
def urlpatterns():
    return [
        url(r'^2fa/auth$', views.LoginView.as_view(), name='wagtail_2fa_auth'),
        url(r'^2fa/devices/$', views.DeviceListView.as_view(), name='wagtail_2fa_device_list'),
        url(r'^2fa/devices/new$', views.DeviceCreateView.as_view(), name='wagtail_2fa_device_new'),
        url(r'^2fa/devices/(?P<pk>\d+)/update$', views.DeviceUpdateView.as_view(), name='wagtail_2fa_device_update'),
        url(r'^2fa/devices/(?P<pk>\d+)/remove$', views.DeviceDeleteView.as_view(), name='wagtail_2fa_device_remove'),
        url(r'^2fa/devices/qr-code$', views.DeviceQRCodeView.as_view(), name='wagtail_2fa_device_qrcode'),
    ]


@hooks.register('construct_main_menu')
def remove_menu_if_unverified(request, menu_items):
    if not request.user.is_verified() and settings.WAGTAIL_2FA_REQUIRED:
        menu_items.clear()
        menu_items.append(
            MenuItem('2FA Setup', reverse('wagtail_2fa_device_list'))
        )


@hooks.register('register_account_menu_item')
def register(request):
    return {
        'url': reverse('wagtail_2fa_device_list'),
        'label': _('Manage your 2FA devices'),
        'help_text': _('Add or remove devices for 2 factor authentication.'),
    }


@hooks.register('register_user_listing_buttons')
def register_user_listing_buttons(context, user):
    yield UserListingButton(
        _('Manage 2FA'),
        reverse('wagtail_2fa_device_list',),
        attrs={'title': _('Edit this user')}, priority=100)



XXXXXXX
XXXXXXX
XXXXXXXXXXX
XXXXXXXXXX
XXXXXXXXX
XXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


from django_otp.plugins.otp_totp.models import TOTPDevice
from django.utils.translation import ugettext_lazy as _


def get_unconfirmed_device(user):
    return(
        TOTPDevice.objects
        .devices_for_user(user, confirmed=False)
        .first())


def new_unconfirmed_device(user):
    delete_unconfirmed_devices(user)
    num = TOTPDevice.objects.filter(user=user).count()
    return TOTPDevice.objects.create(
        name=_("Device #%s") % (num + 1), user=user, confirmed=False)


def delete_unconfirmed_devices(user):
    (TOTPDevice.objects
        .devices_for_user(user, confirmed=False)
        .delete())



import django_otp
from django.conf import settings
from django.contrib.auth.views import redirect_to_login
from django.urls import reverse
from django_otp.middleware import OTPMiddleware as _OTPMiddleware


class VerifyUserMiddleware(_OTPMiddleware):
    _allowed_url_names = [
        'wagtail_2fa_auth',
        'wagtail_2fa_device_list',
        'wagtail_2fa_device_new',
        'wagtail_2fa_device_qrcode',
        'wagtailadmin_login',
        'wagtailadmin_logout',
    ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._allowed_paths = [reverse(n) for n in self._allowed_url_names]

    def process_request(self, request):
        super().process_request(request)
        user = request.user

        if self._require_verified_user(request):
            user_has_device = django_otp.user_has_device(user, confirmed=True)

            if user_has_device and not user.is_verified():
                return redirect_to_login(
                    request.get_full_path(),
                    login_url=reverse('wagtail_2fa_auth'))

            elif not user_has_device and settings.WAGTAIL_2FA_REQUIRED:
                # only allow the user to visit the admin index page and the
                # admin setup page
                return redirect_to_login(
                    request.get_full_path(),
                    login_url=reverse('wagtail_2fa_device_new'))

    def _require_verified_user(self, request):
        user = request.user

        if not user.is_authenticated:
            return False

        # If the user has no access to the admin anyway then don't require a
        # verified user here
        if not (
            user.is_staff or user.is_superuser or
            user.has_perms(['wagtailadmin.access_admin'])
        ):
            return False

        # Allow the user to a fixed number of paths when not verified
        if request.path in self._allowed_paths:
            return False

        # For all other cases require that the user is verfied via otp
        return True




from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.apps import AppConfig


class Wagtail2faConfig(AppConfig):
    name = 'wagtail_2fa'
    label = 'wagtail_2fa'
    verbose_name = "Wagtail 2FA"

    def ready(self):
        if 'wagtail_2fa.middleware.VerifyUserMiddleware' not in settings.MIDDLEWARE:
            raise ImproperlyConfigured(
                "Please add 'wagtail_2fa.middleware.VerifyUserMiddleware' "
                "to django.conf.settings.MIDDLEWARE")

        if not hasattr(settings, 'WAGTAIL_2FA_REQUIRED'):
            settings.WAGTAIL_2FA_REQUIRED = False



from django import forms
from django.utils.translation import ugettext_lazy as _
from django_otp.forms import OTPAuthenticationFormMixin
from django_otp.plugins.otp_totp.models import TOTPDevice


class TokenForm(OTPAuthenticationFormMixin, forms.Form):
    otp_token = forms.CharField(required=True)

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super().__init__(*args, **kwargs)

        self.fields['otp_token'].widget.attrs.update({
            'autofocus': 'autofocus',
            'autocomplete': 'off',
        })

    def clean(self):
        self.clean_otp(self.user)
        return self.cleaned_data


class DeviceForm(forms.ModelForm):
    otp_token = forms.CharField(
        label='OTP token', required=True,
        help_text=_("Enter the numeric code displayed on your device after scanning the QR code"))

    class Meta:
        model = TOTPDevice
        fields = ['name', 'otp_token']

    def __init__(self, user, **kwargs):
        super().__init__(**kwargs)
        self.fields['otp_token'].widget.attrs.update({
            'autofocus': 'autofocus',
            'autocomplete': 'off',
        })

        if self.instance.confirmed:
            del self.fields['otp_token']

        self.user = user

    def clean_otp_token(self):
        token = self.cleaned_data.get('otp_token')

        if token and self.instance.verify_token(token):
            return token

        raise forms.ValidationError(
            _('Invalid token. Please make sure you have entered it correctly.'),
            code='invalid')

    def save(self):
        self.instance.confirmed = True
        self.instance.save()



import re

from setuptools import find_packages, setup

install_requires = [
    'Django>=1.11',
    'Wagtail>=2.1',
    'django-otp>=0.4.3',
    'six>=1.1',
    'qrcode>=5.3',
]

docs_require = [
    'sphinx>=1.4.0',
]

tests_require = [
    'coverage==.4.2',
    'pytest==3.6.0',
    'pytest-cov==2.5.1',
    'pytest-django==3.1.2',

    # Linting
    'isort==4.2.5',
    'flake8==3.0.3',
    'flake8-blind-except==0.1.1',
    'flake8-debugger==1.4.0',
]

with open('README.rst') as fh:
    long_description = re.sub(
        '^.. start-no-pypi.*^.. end-no-pypi', '', fh.read(), flags=re.M | re.S)

setup(
    name='wagtail-2fa',
    version='0.1.0',
    description="Two factor authentication for Wagtail",
    long_description=long_description,
    url='https://github.com/LabD/wagtail-2fa',
    author="Lab Digital",
    author_email="opensource@labdigital.nl",
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require={
        'docs': docs_require,
        'test': tests_require,
    },
    use_scm_version=True,
    entry_points={},
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ],
    zip_safe=False,
)



import qrcode
import qrcode.image.svg
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.views import SuccessURLAllowedHostsMixin
from django.http import HttpResponse
from django.shortcuts import resolve_url
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.functional import cached_property
from django.utils.http import is_safe_url
from django.views.decorators.cache import never_cache
from django.views.decorators.debug import sensitive_post_parameters
from django.views.generic import (
    DeleteView, FormView, ListView, UpdateView, View)
from django_otp import login as otp_login
from django_otp.plugins.otp_totp.models import TOTPDevice

from wagtail_2fa import forms, utils


class LoginView(SuccessURLAllowedHostsMixin, FormView):
    template_name = 'wagtail_2fa/otp_form.html'
    form_class = forms.TokenForm
    redirect_field_name = REDIRECT_FIELD_NAME

    @method_decorator(sensitive_post_parameters())
    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['user'] = self.request.user
        return kwargs

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context[self.redirect_field_name] = self.get_redirect_url()
        return context

    def form_valid(self, form):
        otp_login(self.request, self.request.user.otp_device)
        return super().form_valid(form)

    def get_redirect_url(self):
        """Return the user-originating redirect URL if it's safe."""
        redirect_to = self.request.POST.get(
            self.redirect_field_name,
            self.request.GET.get(self.redirect_field_name, '')
        )
        url_is_safe = is_safe_url(
            url=redirect_to,
            allowed_hosts=self.get_success_url_allowed_hosts(),
            require_https=self.request.is_secure(),
        )
        return redirect_to if url_is_safe else ''

    def get_success_url(self):
        url = self.get_redirect_url()
        return url or resolve_url(settings.LOGIN_REDIRECT_URL)


class DeviceListView(ListView):
    template_name = 'wagtail_2fa/device_list.html'

    def get_queryset(self):
        return (
            TOTPDevice.objects
            .devices_for_user(self.request.user, confirmed=True))


class DeviceCreateView(FormView):
    form_class = forms.DeviceForm
    template_name = 'wagtail_2fa/device_form.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['user'] = self.request.user
        kwargs['instance'] = self.device
        return kwargs

    def form_valid(self, form):
        form.save()
        utils.delete_unconfirmed_devices(form.user)

        if not self.request.user.is_verified():
            otp_login(self.request, form.instance)
        return super().form_valid(form)

    def get_success_url(self):
        return reverse('wagtail_2fa_device_list')

    @cached_property
    def device(self):
        if self.request.method.lower() == 'get':
            return utils.new_unconfirmed_device(self.request.user)
        else:
            return utils.get_unconfirmed_device(self.request.user)


class DeviceUpdateView(UpdateView):
    form_class = forms.DeviceForm
    template_name = 'wagtail_2fa/device_form.html'

    def get_queryset(self):
        return (
            TOTPDevice.objects
            .devices_for_user(self.request.user, confirmed=True))

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['user'] = self.request.user
        return kwargs

    def get_success_url(self):
        return reverse('wagtail_2fa_device_list')


class DeviceDeleteView(DeleteView):
    template_name = 'wagtail_2fa/device_confirm_delete.html'

    def get_queryset(self):
        return (
            TOTPDevice.objects
            .devices_for_user(self.request.user, confirmed=True))

    def get_success_url(self):
        return reverse('wagtail_2fa_device_list')


class DeviceQRCodeView(View):

    def get(self, request):
        device = utils.get_unconfirmed_device(self.request.user)
        img = qrcode.make(device.config_url, image_factory=qrcode.image.svg.SvgImage)
        response = HttpResponse(content_type='image/svg+xml')
        img.save(response)

        return response



XXXXXXXXXXXX
XXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXX
XXXXXXXXXXX

XXXXXX
XXXXXXXXXXXXX

XXXXXX
XXXXXXXXXXXXXX
XXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXX
XXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
